if(!isObject(AnarchyIRCGui))
{
	exec("./AnarchyIRCGui.gui");
	//exec("./Profiles.cs"); //Soon
}

GlobalActionMap.bind(keyboard, "ctrl q", toggleIRC);
function toggleIRC(%x)
{
	if(%x)
	{
		%isAwake = AnarchyIRCGui.isAwake();
		if(%isAwake)
			canvas.popDialog(AnarchyIRCGui);
		else
			canvas.pushDialog(AnarchyIRCGui);
	}
}

function ircClient::onConnected(%this)
{
	%this.isConnected = 1; //We can't send data if we haven't connected
	%this.isAdmin = 0;
	%this.hasAuthOnce = 0;
	%this.send("AUTH " @ $Pref::Player::NetName @ "\r\n");
	AnarchyIRCClient("IRCButtonMode", "CONNECTED");
}

function ircClient::onConnectFailed(%this)
{
	%this.isConnected = 0;
	%this.isAdmin = 0;
	%this.hasAuthOnce = 0;
	warn("IRCClient failed to connect. Use the auth button in the GUI.");
	AnarchyIRCClient("IRCButtonMode", "OFFLINE");
}

function ircClient::onDisconnect(%this)
{
	%this.isConnected = 0; //Make sure they get the correct error
	%this.isAdmin = 0;
	%this.hasAuthOnce = 0;
	warn("IRCClient has disconnected.");
	AnarchyIRCClient("IRCButtonMode", "OFFLINE");
}

function ircClient::onLine(%this, %data)
{
	echo("IRCData: \"" @ %data @ "\"");
	%type = getWord(%data, 0);
	%line = removeWord(%data, 0);
	switch$(%type)
	{
		case "MSG":
			AnarchyIRCClient("RECEIVE", "MSG", %line);

		case "USERS_ON":
			AnarchyIRCClient("UPDATE", "USERS", %line);

		case "USER_ADD":
			AnarchyIRCClient("UPDATE", "USER_ADD", %line);

		case "USER_REMOVE":
			AnarchyIRCClient("UPDATE", "USER_REMOVE", %line);

		case "USER_LIST":
			AnarchyIRCClient("UPDATE", "LIST", %line);

		case "DISCONNECT":
			MessageBoxOK("Disconnect", "You have disconnected from the IRC.\nReason: " @ %line);
			$ircConnection.disconnect();
			$ircConnection.onDisconnect();

		case "AUTH":
			%msg = getWord(%line, 0);
			%isAdmin = getWord(%line, 1);
			switch$(%msg)
			{
				case "SUCCESS":
					AnarchyIRCClient("IRCButtonMode", "ONLINE");
					%this.isAdmin = %isAdmin;
					%this.hasAuthOnce = 1;

				case "FAILED":
					MessageBoxOK("Disconnect", "You have disconnected from the IRC.\nReason: Auth failed.");
					$ircConnection.disconnect();
					$ircConnection.onDisconnect();

				default:
					warn("Unknown AUTH: \"" @ %msg @ "\" - See ircClient::onLine() reference");
			}

		default:
			warn("ircClient::onLine() Unknown command: \"" @ %type @ "\"");
	}
}

package GlobalChat
{
	function MM_AuthBar::blinkSuccess(%this) //Happens when name is changed/auth is success
	{
		Parent::blinkSuccess(%this);
		AnarchyIRCClient("Connect");
	}

	function quit()
	{
		if(isObject($ircConnection) && $ircConnection.isConnected)
		{
			$ircConnection.send("DISCONNECT\r\n");
			$ircConnection.disconnect();
			$ircConnection.onDisconnect();
		}
		return Parent::quit();
	}
};
activatePackage("GlobalChat");

function AnarchyIRCClient(%cmd, %cmd1, %cmd2, %cmd3)
{
	switch$(%cmd)
	{
		case "IRCButtonMode":
			switch$(%cmd1)
			{
				case "CONNECTING":
					AnarchyIRC_OnlinePlayers.clear();
					AnarchyIRC_OnlinePlayersOL.setVisible(0);

					AnarchyIRC_Button.setText("Connecting");
					AnarchyIRC_Button.setColor(getColorF("255 255 150 255"));
					AnarchyIRC_TextInput.setActive(0);
					AnarchyIRC_TextInput.enabled = 0;
					AnarchyIRC_TextInput.setValue("Connecting to server...");
					AnarchyIRC_LeaveButton.setVisible(0);

				case "CONNECTED":
					AnarchyIRC_Button.setText("Connected");
					AnarchyIRC_Button.setColor(getColorF("150 150 255 255"));
					AnarchyIRC_TextInput.setActive(0);
					AnarchyIRC_TextInput.enabled = 0;
					AnarchyIRC_TextInput.setValue("Need to auth before you can chat.");
					AnarchyIRC_LeaveButton.setVisible(1);
					AnarchyIRC_TextOutput.setText(" Welcome to the chat. You will need to auth before you can chat!");

				case "OFFLINE":
					AnarchyIRC_OnlinePlayers.clear();
					AnarchyIRC_OnlinePlayersOL.setVisible(0);

					AnarchyIRC_Window.setText("Anarchy Chat");
					AnarchyIRC_Button.setText("Connect");
					AnarchyIRC_Button.setColor(getColorF("150 255 150 255"));
					AnarchyIRC_TextInput.setActive(0);
					AnarchyIRC_TextInput.enabled = 0;
					AnarchyIRC_TextInput.setValue("You must auth before you can chat.");
					AnarchyIRC_LeaveButton.setVisible(0);

				case "ONLINE":
					AnarchyIRC_Button.setText("Send");
					AnarchyIRC_Button.setColor(getColorF("150 255 150 255"));
					AnarchyIRC_TextInput.setActive(1);
					AnarchyIRC_TextInput.enabled = 1;
					AnarchyIRC_TextInput.setValue("");
					AnarchyIRC_LeaveButton.setVisible(1);
			}

		case "IRCButton": //This should handle everything else
			if(!isObject($ircConnection))
			{
				AnarchyIRCClient("CONNECT");
				return;
			}

			if(!$ircConnection.isConnected)
			{
				AnarchyIRCClient("CONNECT");
				return;
			}

			if(!$ircConnection.hasAuthOnce)
				return;

			AnarchyIRCClient("SEND");

		case "IRCLEAVE":
			if(!isObject($ircConnection))
				return;

			if(!$ircConnection.isConnected)
				return;

			$ircConnection.disconnect();
			$ircConnection.onDisconnect();

		case "RECEIVE":
			switch$(%cmd1)
			{
				case "MSG":
					//if(getWord(AnarchyIRC_TextOutput.position, 1) >= getWord(AnarchyIRC_TextOutput.extent, 1))
					//if(getWord(AnarchyIRC_TextOutput.position, 1) / getWord(AnarchyIRC_TextOutput.extent, 1) < -0.1)
					//	%doResize = 1;

					AnarchyIRC_TextOutput.addText("\n " @ %cmd2, true);
					//if(%doResize)
						AnarchyIRC_TextOutput.resize(0, -getWord(AnarchyIRC_TextOutput.extent, 1), getWord(AnarchyIRC_TextOutput.extent, 0), getWord(AnarchyIRC_TextOutput.extent, 1));
			}

		case "SEND":
			if(!isObject($ircConnection))
			{
				MessageBoxOK("IRC Error", "Sorry, you do not have an IRC connection.\nPlease make sure you: \n1. Have connected to Blockland correctly \n2. Pressed the Connect button.");
				return;
			}

			$ircConnection.send("MSG " @ $Pref::Player::NetName @ ": " @ AnarchyIRC_TextInput.getValue() @ "\r\n");
			AnarchyIRC_TextInput.setValue("");

		case "SENDCMD":
			switch$(%cmd1)
			{
				case "KICK":
					MessageBoxOK("IRC Error", "Sorry, you cannot kick people yet.");

				case "BAN":
					MessageBoxOK("IRC Error", "Sorry, you cannot ban people yet.");
			}

		case "CONNECT":
			if(!isObject(ircClient))
				$ircConnection = new TCPObject(ircClient);

			if($ircConnection.isConnected)
			{
				$ircConnection.disconnect();
				$ircConnection.onDisconnect();
			}
			$ircConnection.connect("86.155.235.157:3330");
			//Don't even think about it...
			AnarchyIRCClient("IRCButtonMode", "CONNECTING");
			AnarchyIRC_OnlinePlayers.clear();
			AnarchyIRC_OnlinePlayersOL.setVisible(0);

		case "UPDATE":
			switch$(%cmd1)
			{
				case "USERS":
					%users = (%cmd2 == 1 ? "1 User" : %cmd2 @ " Users");
					AnarchyIRC_Window.setText("Anarchy Chat - " @ %users);

				case "USER_ADD":
					AnarchyIRC_OnlinePlayers.addRow(getField(%cmd2, 1), getField(%cmd2, 0));
					AnarchyIRC_OnlinePlayers.sort(0, 1);

				case "USER_REMOVE":
					%row = AnarchyIRC_OnlinePlayers.getRowNumByID(%cmd2);
					if(%row != -1)
					{
						AnarchyIRC_OnlinePlayers.removeRow(%row);
						AnarchyIRC_OnlinePlayers.sort(0, 1);
					}
					else
					{
						for(%i = 0; %i < AnarchyIRC_OnlinePlayers.rowCount(); %i++)
						{
							%rowID = AnarchyIRC_OnlinePlayers.getRowText(%i);
							if(%rowID $= %cmd2)
								AnarchyIRC_OnlinePlayers.schedule(0, "removeRow", %i);
						}
					}

				case "LIST": //Same as USER_ADD except this will clear the list
					AnarchyIRC_OnlinePlayers.clear();
					%list = strreplace(%cmd2, "|", "\n");
					for(%i = 0; %i < getLineCount(%list); %i++)
					{
						%line = getLine(%list, %i);
						if(trim(%line) !$= "" && trim(%line) !$= " ")
							AnarchyIRC_OnlinePlayers.addRow(AnarchyIRC_OnlinePlayers.rowCount(), %line);
					}
					AnarchyIRC_OnlinePlayers.sort(0, 1);
			}
	}
}

function AnarchyIRC_OnlinePlayers::onAltClick(%this)
{
	%sel = %this.getSelectedID();
	%selRow = %this.getRowNumByID(%sel);
	%selText = %this.getRowText(%selRow);

	AnarchyIRC_OnlinePlayersOLID.setText("ID: " @ %sel);
	AnarchyIRC_OnlinePlayersOLWindow.setText(%selText);
	if($ircConnection.isAdmin)
	{
		AnarchyIRC_OnlinePlayersOLWindow.resize(0, 0, 142, 124);
		AnarchyIRC_OnlinePlayersOLWindow.center();
	}
	else
	{
		AnarchyIRC_OnlinePlayersOLWindow.resize(0, 0, 142, 88);
		AnarchyIRC_OnlinePlayersOLWindow.center();
	}

	AnarchyIRC_OnlinePlayersOL.setVisible(1);
}

//Thanks to RTB for these
//- GuiControl::shift (moves a gui in the X or Y)
function GuiControl::shift(%this,%x,%y)
{
	%this.position = vectorAdd(%this.position,%x SPC %y);
}

//- GuiControl::center (Centers %this inside %parent)
function GuiControl::center(%this,%parent)
{
	%this.centerX(%parent);
	%this.centerY(%parent);
}

//- GuiControl::getCanvasPosition (returns absolute position of a control on the canvas)
function GuiControl::getCanvasPosition(%this)
{
	%targ = %this;
	%x = getWord(%this.position,0);
	%y = getWord(%this.position,1);
	while(isObject(%targ.getGroup()))
	{
		%parent = %targ.getGroup();
		if(%parent.getName() $= "Canvas")
			return %x SPC %y;
			
		%x += getWord(%parent.position,0);
		%y += getWord(%parent.position,1);
		%targ = %parent;
	}
}

//- GuiControl::centerX (Centers %this inside %parent horizontally)
function GuiControl::centerX(%this,%parent)
{
	if(isObject(%parent))
	{
		%maxArea = getWord(%parent.extent,0);
		%width = getWord(%this.extent,0);
		
		%xPosition = (%maxArea/2)-(%width/2);
		if(%parent $= %this.getGroup())
			%this.position = (%xPosition+getWord(%parent.position,0)) SPC getWord(%this.position,1);
		else
			%this.position = (%xPosition+getWord(%parent.getCanvasPosition(),0)) SPC getWord(%this.getCanvasPosition(),1);
	}
	else
	{
		%parent = %this.getGroup();
		%maxArea = getWord(%parent.extent,0);
		%width = getWord(%this.extent,0);
		
		%xPosition = (%maxArea/2)-(%width/2);
		%this.position = %xPosition SPC getWord(%this.position,1);
	}
}

//- GuiControl::centerY (Centers %this inside %parent vertically)
function GuiControl::centerY(%this,%parent)
{
	if(isObject(%parent))
	{
		%maxArea = getWord(%parent.extent,1);
		%height = getWord(%this.extent,1);

		%yPosition = (%maxArea/2)-(%height/2);
		if(%parent $= %this.getGroup())
			%this.position = getWord(%this.position,0) SPC (%yPosition+getWord(%parent.position,1));
		else
			%this.position = getWord(%this.getCanvasPosition(),0) SPC (%yPosition+getWord(%parent.getCanvasPosition(),1));
	}
	else
	{
		%parent = %this.getGroup();
		%maxArea = getWord(%parent.extent,1);
		%height = getWord(%this.extent,1);
		
		%yPosition = (%maxArea/2)-(%height/2);
		%this.position = getWord(%this.position,0) SPC %yPosition;
	}
}